@@ -2,7 +2,7 @@
 ASGI App using dataclasses module for request/response objects
 """
 
-__version__ = '0.14.0'
+__version__ = '0.14.2'
 
 from apidaora.app import appdaora
 from apidaora.bodies import GZipFactory
@@ -10,7 +10,15 @@ from apidaora.content import ContentType
 from apidaora.exceptions import BadRequestError
 from apidaora.header import Header
 from apidaora.method import MethodType
-from apidaora.responses import html, json, no_content, not_found, text
+from apidaora.middlewares import CorsMiddleware, Middlewares
+from apidaora.responses import (
+    Response,
+    html,
+    json,
+    no_content,
+    not_found,
+    text,
+)
 from apidaora.route.decorator import route
 
 
@@ -27,4 +35,7 @@ __all__ = [
     'route',
     'no_content',
     'not_found',
+    'Response',
+    'Middlewares',
+    'CorsMiddleware',
 ]
