@@ -9,7 +9,6 @@ from typing import (  # type: ignore
     Awaitable,
     Callable,
     Dict,
-    Iterable,
     List,
     Optional,
     Sequence,
@@ -32,10 +31,10 @@ from ..asgi.base import (
     ASGIResponse,
 )
 from ..asgi.responses import (
-    NOT_FOUND_RESPONSE,
-    NO_CONTENT_RESPONSE,
     make_html_response,
     make_json_response,
+    make_no_content_response,
+    make_not_found_response,
     make_text_response,
 )
 from ..asgi.router import Controller, Route
@@ -44,27 +43,33 @@ from ..content import ContentType
 from ..exceptions import BadRequestError, InvalidReturnError
 from ..header import Header
 from ..method import MethodType
+from ..middlewares import (
+    MiddlewareRequest,
+    Middlewares,
+    make_kwargs_from_requst,
+)
 from ..responses import Response
 from .controller_input import controller_input
 
 
 RESPONSES_MAP: Dict[
-    Union[ContentType, HTTPStatus],
-    Union[Callable[..., ASGIResponse], ASGIResponse],
+    Union[ContentType, HTTPStatus], Union[Callable[..., ASGIResponse]],
 ] = {
     ContentType.APPLICATION_JSON: make_json_response,
     ContentType.TEXT_PLAIN: make_text_response,
     ContentType.TEXT_HTML: make_html_response,
-    HTTPStatus.NOT_FOUND: NOT_FOUND_RESPONSE,
-    HTTPStatus.NO_CONTENT: NO_CONTENT_RESPONSE,
+    HTTPStatus.NOT_FOUND: make_no_content_response,
+    HTTPStatus.NO_CONTENT: make_not_found_response,
 }
 
 
 def make_route(
     path_pattern: str,
     method: MethodType,
-    controller: Callable[..., Any],
+    controller: Union[Callable[..., Any], Controller],
     has_content_length: bool = True,
+    route_middlewares: Optional[Union[Middlewares]] = None,
+    options: bool = False,
 ) -> Route:
     ControllerInput = controller_input(controller, path_pattern)
     annotations_info = ControllerInput.__annotations_info__
@@ -79,37 +84,70 @@ def make_route(
         query_dict: ASGIQueryDict,
         headers: ASGIHeaders,
         body: ASGIBody,
+        middlewares: Optional[Middlewares] = None,
     ) -> Dict[str, Any]:
         kwargs: Dict[str, Any]
+        middleware_request: MiddlewareRequest
+        has_middlewares = (middlewares and middlewares.pre_execution) or (
+            route_middlewares and route_middlewares.pre_execution
+        )
+
+        if has_middlewares:
+            middleware_request = MiddlewareRequest()
+
         if annotations_info.has_input:
             if annotations_info.has_path_args:
                 kwargs = {
                     name: path_args.get(name)
                     for name in annotations_path_args.keys()
                 }
+
+                if has_middlewares:
+                    middleware_request.path_args = {
+                        name: deserialize_field(
+                            name, type_, path_args.get(name)
+                        )
+                        for name, type_ in annotations_path_args.items()
+                    }
+
             else:
                 kwargs = {}
 
             if annotations_info.has_query_dict:
                 kwargs.update(
                     {
-                        name: deserialize_query_args(
+                        name: parse_query_args(
                             name, type_, query_dict.get(name)
                         )
                         for name, type_ in annotations_query_dict.items()
                     }
                 )
 
+                if has_middlewares:
+                    middleware_request.query_dict = {
+                        name: deserialize_field(
+                            name,
+                            type_,
+                            parse_query_args(
+                                name, type_, query_dict.get(name)
+                            ),
+                        )
+                        for name, type_ in annotations_query_dict.items()
+                        if name != 'body'
+                    }
+
             if annotations_info.has_headers:
                 headers_map = ControllerInput.__headers_name_map__
-                kwargs.update(
-                    {
-                        name: annotations_headers[name](h_value.decode())  # type: ignore
-                        for h_name, h_value in headers
-                        if (name := headers_map.get(h_name.decode()))  # noqa
-                        in annotations_headers
-                    }
-                )
+                headers_dict: Dict[str, Header] = {
+                    name: annotations_headers[name](h_value.decode())  # type: ignore
+                    for h_name, h_value in headers
+                    if (name := headers_map.get(h_name.decode()))  # noqa
+                    in annotations_headers
+                }
+                kwargs.update(headers_dict)
+
+                if has_middlewares:
+                    middleware_request.headers = headers_dict.copy()
 
             if annotations_info.has_body:
                 if isinstance(body_type, type) and issubclass(
@@ -125,6 +163,21 @@ def make_route(
                 else:
                     kwargs['body'] = make_json_request_body(body, body_type)
 
+            if has_middlewares:
+                middleware_request.body = deserialize_field(
+                    'body', body_type, kwargs['body']
+                )
+
+                if middlewares:
+                    for middleware in middlewares.pre_execution:
+                        middleware(middleware_request)
+
+                if route_middlewares:
+                    for middleware in route_middlewares.pre_execution:
+                        middleware(middleware_request)
+
+                kwargs = make_kwargs_from_requst(middleware_request)
+
             return as_typed_dict(  # type: ignore
                 kwargs, ControllerInput
             )
@@ -137,6 +190,8 @@ def make_route(
         headers: Optional[Sequence[Header]] = None,
         content_type: Optional[ContentType] = ContentType.APPLICATION_JSON,
         return_type_: Any = None,
+        middlewares: Optional[Middlewares] = None,
+        local_route_middlewares: Optional[Middlewares] = route_middlewares,
     ) -> ASGICallableResults:
         while iscoroutine(controller_output):
             controller_output = await controller_output
@@ -144,13 +199,36 @@ def make_route(
         if return_type_ is None and return_type:
             return_type_ = return_type
 
+        has_middlewares = (middlewares and middlewares.post_execution) or (
+            local_route_middlewares and local_route_middlewares.post_execution
+        )
+
+        if has_middlewares and not isinstance(controller_output, Response):
+            controller_output = Response(
+                body=controller_output,
+                status=status,
+                headers=headers,
+                content_type=content_type,
+            )
+
         if isinstance(controller_output, Response):
+            if has_middlewares:
+                if middlewares:
+                    for middleware in middlewares.post_execution:
+                        middleware(controller_output)
+
+                if local_route_middlewares:
+                    for middleware in local_route_middlewares.post_execution:
+                        middleware(controller_output)
+
             return build_asgi_output(
                 controller_output['body'],
                 controller_output['status'],
                 controller_output['headers'],
                 controller_output['content_type'],
                 controller_output.__annotations__.get('body'),
+                middlewares=None,
+                local_route_middlewares=None,
             )
 
         elif isinstance(controller_output, dict):
@@ -159,13 +237,15 @@ def make_route(
             else:
                 body = orjson.dumps(controller_output)
 
-        elif is_dataclass(controller_output):
+        elif (
+            is_dataclass(controller_output)
+            or isinstance(controller_output, tuple)
+            or isinstance(controller_output, list)
+        ):
             body = dataclass_asjson(controller_output)
 
         elif (
-            isinstance(controller_output, tuple)
-            or isinstance(controller_output, str)
-            or isinstance(controller_output, list)
+            isinstance(controller_output, str)
             or isinstance(controller_output, int)
             or isinstance(controller_output, bool)
             or isinstance(controller_output, float)
@@ -179,7 +259,10 @@ def make_route(
             content_length = 0 if has_content_length else None
 
             if content_type is None and status in RESPONSES_MAP:
-                return (RESPONSES_MAP[status],)
+                if headers:
+                    return (RESPONSES_MAP[status](make_asgi_headers(headers)),)
+
+                return (RESPONSES_MAP[status](headers),)
 
             if headers:
                 return RESPONSES_MAP[content_type](  # type: ignore
@@ -210,9 +293,17 @@ def make_route(
             body: ASGIBody,
         ) -> Union[Awaitable[ASGICallableResults], ASGICallableResults]:
             try:
-                kwargs = parse_asgi_input(path_args, query_dict, headers, body)
+                kwargs = parse_asgi_input(
+                    path_args,
+                    query_dict,
+                    headers,
+                    body,
+                    middlewares=self.middlewares,
+                )
                 controller_output = controller(**kwargs)
-                return build_asgi_output(controller_output)
+                return build_asgi_output(
+                    controller_output, middlewares=self.middlewares
+                )
 
             except BadRequestError as error:
                 return send_bad_request_response(
@@ -237,8 +328,16 @@ def make_route(
         annotations_info.has_query_dict,
         annotations_info.has_headers,
         annotations_info.has_body,
+        has_options=options,
     )
-    wrapped_controller.route = route
+    routes = [route]
+
+    if isinstance(controller, Controller) and hasattr(controller, 'routes'):
+        routes = controller.routes + routes
+
+    wrapped_controller.routes = routes
+    wrapped_controller.middlewares = route_middlewares
+
     return route
 
 
@@ -263,7 +362,7 @@ def send_bad_request_response(
     content_length = len(body) if has_content_length else None
 
     return (
-        RESPONSES_MAP[content_type](  # type: ignore
+        RESPONSES_MAP[content_type](
             content_length, HTTPStatus.BAD_REQUEST, make_asgi_headers(headers)
         ),
         body,
@@ -287,7 +386,7 @@ def make_asgi_headers(
     )
 
 
-def deserialize_query_args(
+def parse_query_args(
     name: str, type_: Type[Any], value: Optional[List[str]]
 ) -> Any:
     if value is not None:
@@ -307,4 +406,4 @@ def deserialize_query_args(
         else:
             value = value[0]  # type: ignore
 
-        return deserialize_field(name, type_, value)
+        return value
